package eu.connect.photo.client.backend.rmi;

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.LinkedList;
import java.util.List;

import eu.connect.photo.core.Photo;
import eu.connect.photo.core.PhotoMetaData;
import eu.connect.photo.core.PhotoQuery;
import eu.connect.photo.core.PhotoSharing;
import eu.connect.photo.server.rmi.RMIPhotoSharing;


public class RmiBackEnd implements PhotoSharing {

	public static final String SERVER_URL = "CONNECT_PhotoSharing";
	
	private RMIPhotoSharing server;
	
	public RmiBackEnd() {
		System.setSecurityManager(new RMISecurityManager());
		try {
			Registry r= LocateRegistry.getRegistry("localhost",2048);
			server = (RMIPhotoSharing) r.lookup(SERVER_URL);
			
		} catch (RemoteException e) {
			System.out.println("RMI Backend: An unexpected error occured while contacting the server");
	
		}  catch (NotBoundException e) {
			System.out.println("RMI Backend: Server not found");
			e.printStackTrace();
			
		}
	}
	
	public Photo get(int id) {
		Photo result = null;
		try {
			System.out.println("RMI Backend: Retrieving picture ID(" + id + ")");
			result = server.get(id);
	
		} catch (RemoteException e) {
			System.out.println("RMI Backend: An unexpected error occured while retrieving the picture!");
			e.printStackTrace();
		}
		return result;
	}

	public List<PhotoMetaData> search(PhotoQuery query) {
		List<PhotoMetaData> result = new LinkedList<PhotoMetaData>();
		try {
			System.out.println("RMI Backend: Searching for pictures matching q=" + query);
			result.addAll(server.search(query));
			System.out.println("RMI Backend: " + result.size() + " pictures found!");
			
		} catch (RemoteException e) {
			System.out.println("RMI Backend: An unexpected error occured while retrieving the picture!");
				
		}
		
		return result;
	}

	public void share(Photo photo) {
		try {
			System.out.println("RMI Backend: Sharing new picture entitled \"" + photo.getMetaData().getTitle() + "\"");
			server.share(photo);
			System.out.println("RMI Backend: Upload complete.");
	
		} catch (RemoteException e) {
			System.out.println("RMI Backend: An unexpected error occured while upload the picture!");
				
		}
		
	}

}
