package eu.connect.photo.client.gui;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.table.TableColumnModel;

import eu.connect.photo.core.Photo;
import eu.connect.photo.core.PhotoMetaData;
import eu.connect.photo.core.PhotoQuery;

public class QueryPanel extends JPanel {

	private static final long serialVersionUID = -5236272383546071864L;

	private GUI context;
	
	private JTextField title;
	private JTextField date;
	private JTextField location;
	private JButton refresh;
	private PhotoTableModel results;
	private JTable table;
	private JButton download;


	public QueryPanel(GUI context) {
	
		this.context = context;
		
		this.setBorder((BorderFactory.createTitledBorder(
				BorderFactory.createEtchedBorder(EtchedBorder.LOWERED),
				"Current Photo Query: ")));
		
		this.setLayout(new GridBagLayout());

		// Place the Title and Title label
		JLabel titleLabel = new JLabel("Title:");
		this.add(titleLabel, new GridBagConstraints(0, 0, 1, 1,
				0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(0, 0, 0, 0), 0, 0));
		title = new JTextField();
		title.setPreferredSize(new Dimension(120, 20));
		title.setText("*");
		this.add(title, new GridBagConstraints(0, 1, 1, 1,
				0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(0, 0, 0, 0), 0, 0));
		
		// Place the Date and date label
		JLabel dateLabel = new JLabel("Date:");
		this.add(dateLabel, new GridBagConstraints(1, 0, 1, 1,
				0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(1, 0, 0, 0), 0, 0));
		date = new JTextField();
		date.setPreferredSize(new Dimension(120, 20));
		date.setText("*");
		this.add(date, new GridBagConstraints(1, 1, 1, 1,
				0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(0, 0, 0, 0), 0, 0));
		
		// Place Location and location label
		JLabel locationLabel = new JLabel("Location:");
		this.add(locationLabel, new GridBagConstraints(2, 0, 1, 1,
				0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(2, 0, 0, 0), 0, 0));
		location = new JTextField();
		location.setPreferredSize(new Dimension(120, 20));
		location.setText("*");
		this.add(location, new GridBagConstraints(2, 1, 1, 1,
				0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(0, 0, 0, 0), 0, 0));
		
		refresh = new JButton("Search");
		refresh.addActionListener(new SearchListener(this));
		this.add(refresh, new GridBagConstraints(3, 1, 1, 1,
				0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(0, 0, 0, 0), 0, 0));
		
		

		// Create the table containing the results
		results = new PhotoTableModel();
		table = new JTable(results);
		table.setPreferredSize(new Dimension(200,100));
		//table.setFillsViewportHeight(true);
		
		TableColumnModel resultsColumnModel = table.getColumnModel();
		resultsColumnModel.getColumn(0).setHeaderValue("ID");
		resultsColumnModel.getColumn(0).setPreferredWidth(25);
		resultsColumnModel.getColumn(1).setHeaderValue("Title");
		resultsColumnModel.getColumn(1).setPreferredWidth(50);
		resultsColumnModel.getColumn(2).setHeaderValue("Date");
		resultsColumnModel.getColumn(2).setPreferredWidth(50);
		resultsColumnModel.getColumn(3).setHeaderValue("Location");
		resultsColumnModel.getColumn(3).setPreferredWidth(50);

		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setAutoscrolls(true);
		scrollPane.setPreferredSize(new Dimension(225, 150));
		
		this.add(scrollPane, new GridBagConstraints(0, 2, 4, 3,
				0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(0, 0, 0, 0), 0, 0));
		
		download = new JButton("Download");
		download.addActionListener(new DownloadListener(this));
		this.add(download, new GridBagConstraints(3, 5 , 1, 1,
				0, 0, GridBagConstraints.CENTER, GridBagConstraints.BOTH,
				new Insets(0, 0, 0, 0), 0, 0));
	}
	
	public GUI getContext() {
		return this.context;
	}
	
	
	private static class SearchListener implements ActionListener {

		private QueryPanel context;
		
		public SearchListener(QueryPanel context) {
			this.context = context;
		}
		
		public void actionPerformed(ActionEvent arg0) {
			String titleQuery = context.title.getText().equals("*") ? PhotoQuery.UNSPECIFIED_TITLE : context.title.getText();
			String dateQuery = context.date.getText().equals("*") ? PhotoQuery.UNSPECIFIED_DATE : context.date.getText();
			String locationQuery = context.location.getText().equals("*") ? PhotoQuery.UNSPECIFIED_LOCATION : context.location.getText();
			
			PhotoQuery query = new PhotoQuery(titleQuery, dateQuery, locationQuery);
			List<PhotoMetaData> results = context.getContext().getBackEnd().search(query);
			if (results.isEmpty()) System.out.println("No pictures!");
			context.results.getContents().clear();
			context.results.getContents().addAll(results);
			context.repaint();
		}
		
	}
	
	
	private static class DownloadListener implements ActionListener {
		
		private QueryPanel context;
		
		public DownloadListener(QueryPanel context) {
			this.context = context;
		}
		
		public void actionPerformed(ActionEvent event) {
			
			
			// Get the id of the selected picture
			//if (context.table.RowSelected() == 0 )
			int row = context.table.getSelectedRow();
			int id = (Integer) context.table.getValueAt(row, 0);
			
			// Select where to store the picture
			JFileChooser jfc = new JFileChooser();
			int result = jfc.showSaveDialog(context);
			if (result == JFileChooser.APPROVE_OPTION) {
				String path = jfc.getSelectedFile().getPath();
				try {
					Photo photo = context.getContext().getBackEnd().get(id);
					if (photo != null) {
						photo.storeAs(path);
					} else {
						JOptionPane
						.showMessageDialog(
								null,
								"Error: Inconsistent Photo Server",
								"Unable to found the photo with id=" + id ,
								JOptionPane.ERROR_MESSAGE);
					}
				} catch (IOException e) {
					JOptionPane
					.showMessageDialog(
							null,
							"Error during download",
							"An unexpected IO error occurs while storing the photo on your local disk.",
							JOptionPane.ERROR_MESSAGE);
				}
			}
		}
		
	}

}
