package eu.connect.photo.client.gui;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;

import eu.connect.photo.core.Photo;
import eu.connect.photo.core.PhotoMetaData;

public class SharePanel extends JPanel {

	private static final long serialVersionUID = 8585523994505198530L;

	private GUI context;

	private JTextField file;
	private JButton choose;
	private JTextField title;
	private JTextField date;
	private JTextField location;
	private JButton upload;
	private PhotoPreview preview; 

	public SharePanel(GUI context) {

		this.context = context;

		this.setBorder((BorderFactory.createTitledBorder(BorderFactory
				.createEtchedBorder(EtchedBorder.LOWERED),
				"Uploading a Photo: ")));

		this.setLayout(new GridBagLayout());

		this.setLayout(new GridBagLayout());

		// Place the file and file label, and File Chooser
		JLabel fileLabel = new JLabel("File:");
		// fileLabel.setPreferredSize(new Dimension(75, 20));
		this.add(fileLabel, new GridBagConstraints(0, 0, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));
		file = new JTextField();
		file.setEditable(false);
		// file.setPreferredSize(new Dimension(200, 20));
		file.setText("data/hippo.jpg");
		this.add(file, new GridBagConstraints(1, 0, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));
		choose = new JButton("Choose");
		choose.addActionListener(new ChooseListener(this));
		this.add(choose, new GridBagConstraints(2, 0, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));

		// Place the Title and Title label
		JLabel titleLabel = new JLabel("Title:");
		// titleLabel.setPreferredSize(new Dimension(75, 20));
		this.add(titleLabel, new GridBagConstraints(0, 1, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));
		title = new JTextField();
		// title.setPreferredSize(new Dimension(200, 20));
		title.setText("The Hippo Family");
		this.add(title, new GridBagConstraints(1, 1, 2, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));

		// Place the apercu of the picture
		preview = new PhotoPreview();
		// preview.setPreferredSize(new Dimension(200, 150));
		this.add(preview, new GridBagConstraints(3, 0, 3, 4, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));

		// Place the Date and date label
		JLabel dateLabel = new JLabel("Date:");
		// dateLabel.setPreferredSize(new Dimension(75, 20));
		this.add(dateLabel, new GridBagConstraints(0, 2, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));
		date = new JTextField();
		// date.setPreferredSize(new Dimension(200, 20));
		date.setText("June 2010");
		this.add(date, new GridBagConstraints(1, 2, 2, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));

		// Place Location and location label
		JLabel locationLabel = new JLabel("Location:");
		// locationLabel.setPreferredSize(new Dimension(75, 20));
		this.add(locationLabel, new GridBagConstraints(0, 3, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));
		location = new JTextField();
		// location.setPreferredSize(new Dimension(200, 20));
		location.setText("Africa");
		this.add(location, new GridBagConstraints(1, 3, 2, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));

		upload = new JButton("Upload");
		upload.addActionListener(new UploadListener(this));
		this.add(upload, new GridBagConstraints(3, 4, 1, 1, 0, 0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						0, 0, 0, 0), 0, 0));

	}

	public GUI getContext() {
		return this.context;
	}

	private static class UploadListener implements ActionListener {

		private SharePanel context;

		public UploadListener(SharePanel context) {
			this.context = context;
		}

		public void actionPerformed(ActionEvent arg0) {
			String title = context.title.getText();
			String date = context.date.getText();
			String location = context.location.getText();

			File file = new File(context.file.getText());

			try {
				Photo photo = new Photo(
						new PhotoMetaData(title, date, location), file);
				context.getContext().getBackEnd().share(photo);

			} catch (FileNotFoundException e) {
				JOptionPane
						.showMessageDialog(
								null,
								"Error during Upload",
								"The file you specified cannot be found on disk.\nPlease use the choose button to ensure the path that you specify is correct.",
								JOptionPane.ERROR_MESSAGE);

			} catch (IOException e) {
				JOptionPane
						.showMessageDialog(
								null,
								"Error during Upload",
								"An unexpected IO error occurs while reading the file from the disk.\nThe upload operation has been aborted.",
								JOptionPane.ERROR_MESSAGE);
			}

		}

	}

	private static class ChooseListener implements ActionListener {

		private SharePanel context;

		public ChooseListener(SharePanel context) {
			this.context = context;
		}

		public void actionPerformed(ActionEvent arg0) {
			JFileChooser fc = new JFileChooser();
			int returnVal = fc.showOpenDialog(context);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				File file = fc.getSelectedFile();
				context.preview.setImage(file);
				context.file.setText(file.getPath());
				context.repaint();
			}
		}

	}

}
