package eu.connect.photo.core;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;

public class Photo implements Serializable {

	private static final long serialVersionUID = 2253180868505221142L;

	private PhotoMetaData metaData;
	private byte[] data;

	/**
	 * Create a new photo
	 * 
	 * @param id
	 *            the id of the picture
	 * @throws IOException
	 * @throws FileNotFoundException
	 * @throws ClassNotFoundException
	 */ 
	public Photo(PhotoMetaData metaData, File file)
			throws FileNotFoundException, IOException {
		this.metaData = metaData;
		FileInputStream fis = new FileInputStream(file);
		data = new byte[fis.available()];
		fis.read(data); 
	}

	/**
	 * Create a Photo from the metadata and the byte of the file
	 * @param metadata the metadata of the file
	 * @param data the raw data of the file
	 */
	public Photo(PhotoMetaData metadata, byte[] data) {
		this.metaData = metadata;
		this.data = data;
	}

	/**
	 * @return the meta data describing the photo
	 */
	public PhotoMetaData getMetaData() {
		return metaData;
	}

	/**
	 * Store the file on the disk, the meta data are lost and must be saved
	 * separately.
	 * 
	 * @param fileToCreate
	 *            the path to the file to create
	 * @throws IOException
	 *             when some erros occurs while writing down the file
	 */
	public void storeAs(String fileToCreate) throws IOException {
		FileOutputStream fos = new FileOutputStream(new File(fileToCreate));
		fos.write(data);
	}

	
	/**
	 * Return the raw data contained in the file
	 * @return the raw data contained in the file
	 */
	public byte[] getRawData() {
		return data;
	}

}
