package eu.connect.photo.core;

import java.io.Serializable;

public class PhotoQuery implements Serializable {

	private static final long serialVersionUID = -4688782333294605L;
	
	public static final int UNSPECIFIED_ID = -1;
	public static final String UNSPECIFIED_TITLE = "__#__UNSPECIFIED_TITLE__#__";
	public static final String UNSPECIFIED_DATE = "__#__UNSPECIFIED_DATE__#__";
	public static final String UNSPECIFIED_LOCATION = "__#__UNSPECIFIED_LOCATION__#__";
 
	private int id;
	private String title;
	private String date;
	private String location;
	
	
	/**
	 * Create an unspecified query that all meta-data match
	 */
	public PhotoQuery() {
		this.id = UNSPECIFIED_ID;
		this.title = UNSPECIFIED_TITLE;
		this.date = UNSPECIFIED_DATE;
		this.location = UNSPECIFIED_LOCATION;
	}
	
	/**
	 * Create a query over the description, i.e. the title, the location and the date.
	 * @param title The expected title of the picture or the UNSPECIFIED_TITLE constant if this field is not constrained
	 * @param date the expected date of picture or the UNSPECIFIED_DATE constant  if this field is not constrained
	 * @param location the expected location of the picture or the UNSPECIFIED_LOCATION this field is not constrained
	 */
	public PhotoQuery(String title, String date, String location) {
		super(); 
		this.id = -1;
		this.title = title;
		this.date = date;
		this.location = location;
	}
	
	/**
	 * Create a query over the id of the picture.
	 * @param photoId the expected photo id of the picture. Using the special 
	 */
	public PhotoQuery(int photoId) {
		this.id = photoId;
		this.title = UNSPECIFIED_TITLE;
		this.date = UNSPECIFIED_DATE;
		this.location = UNSPECIFIED_LOCATION;
	}
	


	public int getId() {
		return this.id;
	}
	
	public void setId(int id) {
		this.id = id;
	}
	
	public boolean isIdUnspecified() {
		return id == UNSPECIFIED_ID;
	}

	public String getTitle() {
		return title;
	}
	
	public void setTitle(String title) {
		this.title = title;
	}
	
	public boolean isTitleUnspecified() {
		return title.equals(UNSPECIFIED_TITLE);
	}
	
	public String getDate() {
		return date;
	}
	
	public void setDate(String date) {
		this.date = date;
	}
	
	public boolean isDateUnspecified() {
		return date.equals(UNSPECIFIED_DATE);
	}
	
	public String getLocation() {
		return location;
	}
	
	public void setLocation(String location) {
		this.location = location;
	}
	
	public boolean isLocationUnspecified() {
		return location.equals(UNSPECIFIED_LOCATION);
	}
	
	
	public String toString() {
		String idView = isIdUnspecified() ? "*"  : new Integer(id).toString();
		String titleView = isTitleUnspecified() ? "*" : title;
		String dateView = isDateUnspecified() ? "*" : date;
		String locationView = isLocationUnspecified() ? "*" : location;
		return String.format("[id=%s;t=%s;d=%s;l=%s]", idView, titleView, dateView, locationView);
	}
	
	
}
